//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DGraphNode
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Wed Nov 24 10:13:29 CET 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DGraphNode.h,v 1.23 2001/07/15 12:39:24 probert Exp $
//-----------------------------------------------------------------------------

#ifndef __G3DGraphNode_h_INCLUDE
#define __G3DGraphNode_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/GeometryKit.h>

#import <RenderKit/G3DTypes.h>
#import <RenderKit/G3DDefines.h>
#import <RenderKit/G3DRenderEngine.h>
#import <RenderKit/G3DSceneCoding.h>

@class G3DFrustum;
@class G3DTransform;
@class G3DGroup;
@class G3DBVol;

/*
 * The purpose of sorting is to group nodes together, which share some
 * common state characteristics or texture objects!
 * Textured objects have the highest priority, which is computed using the
 * tex obj ID
 *
 */

#define G3D_PRIORITYMASK(X)	(1<<(X))

#define G3D_ENABLE_PRIORITY(X)  renderPriority.priority |= (1<<X)
#define G3D_DISABLE_PRIORITY(X) renderPriority.priority &= ~(1<<X)
#define G3D_HAS_PRIORITY(X)     (renderPriority.priority & (1<<X))

#define G3D_PRIORITY_BLENDING    0
#define G3D_PRIORITY_GENERIC     1
#define G3D_PRIORITY_SMOOTH      2
#define G3D_PRIORITY_COLMATERIAL 3
#define G3D_PRIORITY_LIGHTING    4
#define G3D_PRIORITY_TEXTURING   5

typedef struct _G3D_SortPriority {
  int priority;
  int value;
} G3D_SortPriority;

@interface G3DGraphNode : NSObject <NSCopying, NSCoding, G3DSceneCoding>
{
  @public
  
  BOOL _transfIsDirty;
  int level;

  @protected
    
  G3DGraphNode *parent;

  id delegate;
  id userData;

  G3DTransform *transformation;
  G3DBVol *bVolume;

  NSString *name;
  BOOL isVisible;

  unsigned cullDepth;
  G3D_SortPriority renderPriority;

  BOOL _recentlyUpdated;
  BOOL _bVolIsDirty;
  BOOL _renderAsBVol;
  BOOL _hasBVolume;
  
  G3DMatrix4f *_fwdT; 	// world -> position

  BOOL _shouldVFCull;
  BOOL _isCompiled;

  BOOL _shouldNotifyBeforeRendering;
  BOOL _shouldNotifyAfterRendering;
  BOOL _shouldNotifyAfterUpdating;
  BOOL _hasDelegate;
}

//-----------------------------------------------------------------------------
// Render delegate
//-----------------------------------------------------------------------------

- (void)setDelegate:(id)aDelegate;
- (id)delegate;

//-----------------------------------------------------------------------------
// Rendering and traversing the scenegraph
//-----------------------------------------------------------------------------

- (void)renderWithMask:(unsigned)mask frustum:(G3DFrustum *)fr;
  // Abstract method, raises a G3DGraphNodeDesignFlawException if invoked.

- (void)setShouldVFCull:(BOOL)yn;
- (BOOL)shouldVFCull;

- (void)setCullDepth:(unsigned)dep;
- (unsigned)cullDepth;

- (void)setIsVisible:(BOOL)flag;
- (BOOL)isVisible;
- (void)toggleVisibility;

//-----------------------------------------------------------------------------
// The parents...
//-----------------------------------------------------------------------------

- (G3DGraphNode *)parent;

- (void)setParent:(G3DGraphNode *)aParent;
  // Gets invoked by a aParent when the node is being added as a child. You never should invoke it directly!

//-----------------------------------------------------------------------------
// User data
//-----------------------------------------------------------------------------

- (id)userData;
- (void)setUserData:(id)data;
  // Additional information that is stored in an 'attached' object.

//-----------------------------------------------------------------------------
// Transformation
//-----------------------------------------------------------------------------

- (void)update;
- (void)updateIfNeeded;
  // Updates the model transformation and perhaps other ivars recursively (downwards). You never should invoke those methods directly!

- (void)updateWithTransformation:(G3DMatrix4f *)t;
  // Gets invoked while updating the graph. You never should invoke this method directly! Sends graphNodeDidUpdate if the delegate responds to it.
 
- (G3DMatrix4f *)modelTransformation;
  // The model transformation from the root node down to the current graphnode.

- (G3DTransform *)transformation;
- (void)setTransformation:(G3DTransform *)transf;
  // Sets the transformation in the local coordinate system. When being set or modified, it marks itself as dirty.

//-----------------------------------------------------------------------------
// Optimisiation
//-----------------------------------------------------------------------------

- (void)compile;
  // Abstract method, does nothing.

- (BOOL)isCompiled;
  // Compilation is lossy! Once a subgraph is compiled this cannot be reverted!

- (void)updatePriority;
- (G3D_SortPriority*)renderPriority;

//-----------------------------------------------------------------------------
// Node information
//-----------------------------------------------------------------------------

- (void)setName:(NSString *)aName;
- (NSString *)name;

- (int)level;
  // Lazily returns the level of the current node.

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;
  // Copying is deep!

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

@interface G3DGraphNode (BoundingVolumeHandling)

- (G3DBVol *)boundingVolume;
  // Returns the current bounding volume, after updating it (if needed)

- (void)setNeedsUpdate:(BOOL)yn;
  // Marks its parent as dirty, if yn

- (BOOL)needsUpdate;
- (BOOL)recentlyUpdated;

- (void)setRecentlyUpdated:(BOOL)yn;
  // Needed for optimised bVolume updating. Never invoke this method!

- (void)setRenderAsBVolume:(BOOL)flag;
- (BOOL)renderAsBVolume;

- (void)setHasBVolume:(BOOL)yn;
- (BOOL)hasBVolume;

- (void)updateBVolume;
  // Abstract method,does nothing!
 
- (void)updateBVolumeIfNeeded;
  // Invokes updateBVolume if marked as dirty

- (BOOL)collidesWithNode:(G3DGraphNode *)anotherNode;

@end

@interface NSObject (GraphNodeDelegates)

- (void)graphNodeWillRender:(id)sender;
- (void)graphNodeDidRender:(id)sender;
- (void)graphNodeDidCull:(id)sender;
- (void)graphNodeDidUpdate:(id)sender;

@end

extern NSString *G3DGraphNodeException;
extern NSString *G3DGraphNodeOptimisationException;
extern NSString *G3DGraphNodeCompilationException;
extern NSString *G3DGraphNodeDesignFlawException;
extern NSString *G3DGraphNodeCollisionDetectionException;

#endif
