//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DGroup
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DGroup.h,v 1.16 2001/07/07 15:35:47 probert Exp $
//-----------------------------------------------------------------------------

/*
 * Groups contain only structural information about the scenegraph. All
 * geometric information that will be drawn iscontained in leaf nodes.
 * To gain more speed, group nodes can be optimised, that is the scenegraph's
 * structure gets simplified if possible.
 */

#ifndef __G3DGroup_h_INCLUDE
#define __G3DGroup_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/GeometryKit.h>

#import <RenderKit/G3DTypes.h>
#import <RenderKit/G3DGraphNode.h>

@class G3DNodeList;

@interface G3DGroup : G3DGraphNode <NSCopying, NSCoding, G3DSceneCoding>
{
  @public
  NSMutableArray *children;

  @protected
  BOOL _isOptimised;
  BOOL _needsOptimisation;
  BOOL _allowReordering;

  G3DNodeList *_flattenedNodes;

  int _children;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (id)group;
+ (id)groupWithCapacity:(unsigned int)cap allowReordering:(BOOL)yn;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)init;
- (id)initWithCapacity:(unsigned int)cap allowReordering:(BOOL)yn;

//-----------------------------------------------------------------------------
// G3DGroupingNode methods
//-----------------------------------------------------------------------------

- (NSArray *)children;
- (NSArray *)leaves;

- (void)addChild:(G3DGraphNode *)aChild;
  // Raises a G3DGroupException if aChild is already prebound!

- (void)insertChild:(G3DGraphNode *)aChild atIndex:(int)ind;
  // Raises a G3DGroupException if aChild is already prebound!

- (int)childrenCount;
- (G3DGraphNode *)childAtIndex:(int)index;
- (NSEnumerator *)childEnumerator;

- (void)removeChildAtIndex:(int)ind;
- (void)removeChild:(G3DGraphNode *)aChild;
- (void)removeAllChildren;

- (void)replaceChildAtIndex:(int)ind withChild:(G3DGraphNode *)aChild;

//-----------------------------------------------------------------------------
// Sorting
//-----------------------------------------------------------------------------

- (void)sortUsingFunction:(int(*)(id,id,void*))comparator context:(void *)cxt;
- (void)sortUsingSelector:(SEL)comparator;

- (BOOL)allowReordering;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;
  // Copy is deep!

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

@interface G3DGroup (GraphOptimisation)

- (void)compile;
  // First invokes optimise and then compiles itself and all its children. This is LOSSY!

- (void)optimise;
- (void)optimiseIfNeeded;
  // Optimisations means bounding volume updating, transformation precalculations etc.

- (BOOL)isOptimised;

- (BOOL)canBeFlattened;
  // Returns YES. Subclasses might override it if needed.

- (void)updatePriority;
  // The group node takes the highest priority of its children.

@end

extern NSString *G3DGroupException;

extern NSString *G3DLightRemovedNotification;
extern NSString *G3DLightAddedNotification;

#endif




