//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DIVtxArray
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DIVtxArray.h,v 1.7 2001/01/05 20:50:52 robert Exp $
//-----------------------------------------------------------------------------

/*
 * This class is used by G3DShape to store all the geometry. It is a wrapper for
 * an interleaved vertex array.
 *
 * Attention:	you can not use all accessor methods all the time, although
 * 		there are no assertions builtin! Their use is depending on the
 *		current vertex geometry format!
 */

#ifndef __G3DIVtxArray_h_INCLUDE
#define __G3DIVtxArray_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DTypes.h>
#import <RenderKit/G3DAttributeNode.h>
#import <RenderKit/G3DGeometryBackend.h>

@class G3DShape;

// For more information about vertex array formats see OGL 1.1 Ref P.77

typedef enum {
  G3D_V2 = GL_V2F,
  G3D_V3 = GL_V3F,
  G3D_C3V3 = GL_C3F_V3F,
  G3D_N3V3 = GL_N3F_V3F,
  G3D_C4N3V3 = GL_C4F_N3F_V3F,
  G3D_T2V3 = GL_T2F_V3F,
  G3D_T4V4 = GL_T4F_V4F,
  G3D_T2C3V3 = GL_T2F_C3F_V3F,
  G3D_T2N3V3 = GL_T2F_N3F_V3F,
  G3D_T2C4N3V3 = GL_T2F_C4F_N3F_V3F,
  G3D_T4C4N3V4 = GL_T4F_C4F_N3F_V4F
} G3DArrayFormat;

@interface G3DIVtxArray : G3DAttributeNode <NSCoding,NSCopying,G3DGeometryBackend,G3DSceneCoding>
{
  int vertices;
  int stride;

  int format; // G3DArrayFormat
  int geometryMode; // G3DGeometryMode

  float *array; // The interleaved vertex array

  G3DRange col; // Size and offset of the various subarrays
  G3DRange ver;
  G3DRange nor;
  G3DRange tex;

  BOOL isLocked;
  int _vChunkSize;

  G3DShape *shape;	// Gets informed to update its bVolume, if needed
  
  @private
  BOOL _isInitialised;
}

//-----------------------------------------------------------------------------
// Class methods 
//-----------------------------------------------------------------------------

+ (G3DIVtxArray *)triangleArrayWithData:(const float *)a count:(unsigned)n;
  // Expects an array in format G3D_T4C4N3V4, mode G3D_TRIANGLES

+ (G3DIVtxArray *)triangleStripArrayWithData:(const float *)a count:(unsigned)n;
  // Expects an array in format G3D_T4C4N3V4, mode G3D_TRIANGLE_STRIP

//-----------------------------------------------------------------------------
// Init
//-----------------------------------------------------------------------------

- (id)initWithMode:(G3DGeometryMode)mode format:(G3DArrayFormat)fmt count:(int)cnt array:(const float *)varr;
  // If varr is not NULL it copies the provided array in the specified vertex format. Otherwise it allocates a new array of the needed size and zeroes it. cnt has to be > 0!

//-----------------------------------------------------------------------------
// Drawing
//-----------------------------------------------------------------------------

- (void)drawGeometry;
  // Is invoked by its G3DShape to render it when no display list is to be used!

//-----------------------------------------------------------------------------
// Accessor Methods
//-----------------------------------------------------------------------------

- (void)precompile;
  // Makes the array locked, if possible

- (int)vertexCount;

- (float *)array;

- (G3DArrayFormat)format;
  // The interleaved vertex array format as specified in OpenGL 1.1

- (G3DGeometryMode)geometryMode;

- (G3DRange)colour;
- (G3DRange)normal;
- (G3DRange)vertex;
- (G3DRange)texture;

- (void)getMin:(float *)min max:(float *)max;
  // Fills the 2 float arrays of size ver.length with the 'virtual' min and the max of the space that is occupied by this vertex array. This canbeused to calculate an AABB.

- (void)setShape:(G3DShape *)aShape;
  // Never invoke this directly! This is private!

//-----------------------------------------------------------------------------
// Geometry
//-----------------------------------------------------------------------------

- (void)setCoordinate:(float *)coord atIndex:(int)index;
- (float *)coordinateAtIndex:(int)index;

- (void)setCoordinates:(float *)coords atIndex:(int)index count:(int)cnt;
- (void)setCoordinates:(float *)coords range:(NSRange)rng;

- (void)setNormal:(float *)coord atIndex:(int)index;
- (float *)normalAtIndex:(int)index;

- (void)setNormals:(float *)coords atIndex:(int)index count:(int)cnt;
- (void)setNormals:(float *)coords range:(NSRange)rng;

//-----------------------------------------------------------------------------
// Colours
//-----------------------------------------------------------------------------

- (void)setColour:(float *)col atIndex:(int)index;
- (float *)colourAtIndex:(int)index;

- (void)setColours:(float *)cols atIndex:(int)index count:(int)cnt;
- (void)setColours:(float *)cols range:(NSRange)rng;

//-----------------------------------------------------------------------------
// Texture coordinates
//-----------------------------------------------------------------------------

- (void)setTexCoord:(float *)coord atIndex:(int)index;
- (float *)texCoordAtIndex:(int)index;

- (void)setTexCoords:(float *)coords atIndex:(int)index count:(int)cnt;
- (void)setTexCoords:(float *)coords range:(NSRange)rng;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DVtxArrayException;
extern NSString *G3DVtxArrayFormatException;
extern NSString *G3DVtxArrayLockingException;

#endif




