//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DLight
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DLight.h,v 1.13 2001/07/15 12:39:24 probert Exp $
//-----------------------------------------------------------------------------

/*
 * G3DLight is used for dynamic lighting, done with GL directly. So unless you
 * have a fast machine and/or T&L acceleration on you graphics card, make
 * careful use of lights, they slow down rendering a lot!
 *
 * Important: Don't set the light's transformation, it's already in use! Rather
 * put the light in a grouping node
 *
 */

#ifndef __G3DLight_h_INCLUDE
#define __G3DLight_h_INCLUDE

#import <Foundation/Foundation.h>

#import <RenderKit/G3DLeaf.h>

@class G3DColour;
@class G3DVector4f;

typedef enum {
  G3D_PointLight,
  G3D_DistantLight,
  G3D_SpotLight
} G3D_LightType;

@interface G3DLight : G3DLeaf <NSCopying,NSCoding,G3DSceneCoding>
{
  @public
    
  int lightType;                // G3D_LightType
  int _lhandle;

  BOOL isSwitchedOn;

  float _pos[4];                // direction/position, depending on w
  float _dir[4];                // direction/position, depending on w
  float _trnPos[4];             // direction/position in world coords
  float _trnDir[4];             // direction/position in world coords

  @protected
    
  float attenuation;
  float linearAttenuation;
  float quadraticAttenuation;

  float spotExponent;
  float spotCutOff;

  G3DColour *specular;
  G3DColour *diffuse;
  G3DColour *ambient;
  
  BOOL isGlobal;
  
  BOOL _firstTime;
  BOOL _lightHasChanged;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DLight *)distantLightWithDirection:(G3DVector3f *)vec;
  // A light with no falloff over distance, located at infinity.

+ (G3DLight *)pointLightWithPosition:(G3DVector3f *)p;
  // A light from a single point with falloff over distance.

+ (G3DLight *)spotLightWithPosition:(G3DVector3f *)pos direction:(G3DVector3f *)d;
  // A light from a single point with falloff over distance and angle.

//-----------------------------------------------------------------------------
// Rendering and Position
//-----------------------------------------------------------------------------

- (void)setDirection:(G3DVector3f *)dir;
- (void)getDirection:(G3DVector3f **)dir;

- (void)setPosition:(G3DVector3f *)pos;
- (void)getPosition:(G3DVector3f **)pos;

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (int)lightHandle;
  // Returns a valid GL handle if is in use, -1 otherwise

- (G3D_LightType)lightType;

- (BOOL)isSwitchedOn;
- (void)setIsSwitchedOn:(BOOL)flag;

- (BOOL)isGlobal;
- (void)setIsGlobal:(BOOL)flag;

- (G3DColour *)specular;
- (void)setSpecular:(G3DColour *)aColour;

- (G3DColour *)diffuse;
- (void)setDiffuse:(G3DColour *)aColour;

- (G3DColour *)ambient;
- (void)setAmbient:(G3DColour *)aColour;

- (float)spotExponent;
- (void)setSpotExponent:(float)exp;

- (float)spotCutOff;
- (void)setSpotCutOff:(float)cutOff;

- (float)attenuation;
- (void)setAttenuation:(float)val;

- (float)linearAttenuation;
- (void)setLinearAttenuation:(float)val;

- (float)quadraticAttenuation;
- (void)setQuadraticAttenuation:(float)val;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DLightException;

#endif







