//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DTexture
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DTexture.h,v 1.14 2001/01/05 20:50:55 robert Exp $
//-----------------------------------------------------------------------------

/*
 * Description:
 *
 * Width and height must have the format 2^m + 2^b, where
 *
 *  m < 0 and b = border 
 *
 * Be sure to make all the needed customisations before you set the texture
 * object to a shape. Once this is done, the texture is initialised and cannot
 * be modified anymore!
 *
 */

#ifndef __G3DTexture_h_INCLUDE
#define __G3DTexture_h_INCLUDE

#import <Foundation/Foundation.h>

#import <RenderKit/G3DAttributeNode.h>
#import <RenderKit/G3DRenderEngine.h>

typedef enum {
  G3D_LUMINANCE_INT = 1,
  G3D_LUMINANCE_ALPHA_INT = 2,
  G3D_RGB_INT = 3,
  G3D_RGBA_INT = 4
} G3D_TexFmtInt;

typedef enum {
  G3D_RGB = GL_RGB,
  G3D_RGBA = GL_RGBA,
  G3D_RED = GL_RED,
  G3D_GREEN = GL_GREEN,
  G3D_BLUE = GL_BLUE,
  G3D_ALPHA = GL_ALPHA,
  G3D_LUMINANCE = GL_LUMINANCE,
  G3D_LUMINANCE_ALPHA = GL_LUMINANCE_ALPHA
} G3D_TexFmt;

typedef enum {
  G3D_BYTE = GL_BYTE,
  G3D_UBYTE = GL_UNSIGNED_BYTE,
  G3D_SHORT = GL_SHORT,
  G3D_USHORT = GL_UNSIGNED_SHORT,
  G3D_INT = GL_INT,
  G3D_UINT = GL_UNSIGNED_INT,
  G3D_FLOAT = GL_FLOAT,
  G3D_BITMAP = GL_BITMAP,
} G3D_TexType;

typedef enum {
  G3D_MODULATE = GL_MODULATE,
  G3D_DECAL = GL_DECAL,
  G3D_BLEND = GL_BLEND,
  G3D_REPLACE = GL_REPLACE
} G3D_TexEnvMode;

typedef enum {
  G3D_CLAMP = GL_CLAMP,
  G3D_REPEAT = GL_REPEAT,
} G3D_TexWrap;

typedef enum {
  G3D_MAG_NEAREST = GL_NEAREST,
  G3D_MAG_LINEAR = GL_LINEAR
} G3D_TexMagFilter;

typedef enum {
  G3D_MIN_NEAREST = GL_NEAREST,
  G3D_MIN_LINEAR = GL_LINEAR,
  G3D_MIN_NMN = GL_NEAREST_MIPMAP_NEAREST,
  G3D_MIN_NML = GL_NEAREST_MIPMAP_LINEAR,
  G3D_MIN_LMN = GL_LINEAR_MIPMAP_NEAREST,
  G3D_MIN_LML = GL_LINEAR_MIPMAP_LINEAR
} G3D_TexMinFilter;

typedef struct _rk_wrappong {
  int s;
  int t;
} G3DWrapping;

typedef struct _rk_offset {
  int x;
  int y;
} G3DOffset;

typedef struct _rk_tex_geometry {
  G3DOffset offset;
  int width;
  int height;
  int border;
} G3DTexGeometry;

@class G3DColour;

@interface G3DTexture : G3DAttributeNode <NSCoding, NSCopying, G3DSceneCoding>
{    
  NSMutableData *imageData;
  unsigned int texName;
  unsigned int texObj[1];

  G3DTexGeometry texGeometry;
  G3DWrapping wrap;

  int ifmt;              // G3D_TexFmtInt
  int fmt;               // G3D_TexFmt
  int type;              // G3D_TexType
  int mode;              // G3D_TexEnvMode
  int mag;               // G3D_TexMagFilter
  int min;               // G3D_TexMinFilter

  G3DColour *blendcol;
  G3DColour *bordercol;

  double priority;

  BOOL _isBound;
  BOOL _createMipmaps;    // Default is NO
  int _components;        // 1,3,4 ...

  NSString *_texFileName;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DTexture *)textureWithContentsOfFile:(NSString *)aPath;

//-----------------------------------------------------------------------------
// Init and free
//-----------------------------------------------------------------------------

- (id)initWithContentsOfFile:(NSString *)aPath;
  // Does NOT make use of the AppKit, therefore only BMPs and TGAs are currently supported. If loading fails, an exception is raised.

//-----------------------------------------------------------------------------
// Texture creation
//-----------------------------------------------------------------------------

- (void)bindTextureIfNeeded;
// This method finally generates the texture object! This has to be done before using the texture!

- (unsigned int)handle;

//-----------------------------------------------------------------------------
// Texture handling
//-----------------------------------------------------------------------------

- (void)scaleWithFormat:(G3D_TexFmt)fmt type:(G3D_TexType)tp geometry:(G3DTexGeometry)gm;
// Scales the texture using gluScaleImage(). Raises an exception if it fails!

- (void)addSubImage:(void*)img level:(int)l format:(G3D_TexFmt)fmt type:(G3D_TexType)tp geometry:(G3DTexGeometry)gm;
// Uses glTexSubImage2D() to replace an existing rect of a texture

- (void)copySubImageAtLevel:(int)l geometry:(G3DTexGeometry)gm to:(G3DOffset)tg;
// Uses glCopyTexSubImage2D() to replace an existing rect of a texture

//-----------------------------------------------------------------------------
// Texture filtering and environment handling
//-----------------------------------------------------------------------------

- (void)setBuildMipmaps:(BOOL)yn;
- (BOOL)buildMipmaps;

- (void)setMagFilter:(G3D_TexMagFilter)magf;
- (G3D_TexMagFilter)magFilter;

- (void)setMinFilter:(G3D_TexMinFilter)minf;
- (G3D_TexMinFilter)minFilter;

- (G3D_TexEnvMode)textureEnvironmentMode;
- (void)setTextureEnvironmentMode:(G3D_TexEnvMode)md;
// Applies if textureEnvironment == G3D_MODE

- (void)setBlendColour:(G3DColour *)aCol;
- (G3DColour *)blendColour;
// Applies if textureEnvironment == G3D_COLOUR

- (void)setTextureWrapping:(G3DWrapping)wr;
- (G3DWrapping)textureWrapping;

- (void)setBorderColour:(G3DColour *)bcol;
- (G3DColour *)borderColour;

- (void)setPriority:(double)p;
- (double)priority;
// The priority lies between 0.0 and 1.0

- (G3DTexGeometry *)texGeometry;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DTextureScalingException;
extern NSString *G3DTextureMipmapException;
extern NSString *G3DTextureImageSizeException;
extern NSString *G3DTextureFormatException;
extern NSString *G3DTextureReadingException;

#endif
