//-----------------------------------------------------------------------------
// Project            	RenderKit
// Class		G3DTransform
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The RenderKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DTransform.h,v 1.13 2001/01/05 20:50:55 robert Exp $
//-----------------------------------------------------------------------------

/*
 * A Transformation node executes a scale, rotation and transformation
 * operation on the stack. Skewing is not supported!
 *
 */

#ifndef __G3DTransform_h_INCLUDE
#define __G3DTransform_h_INCLUDE

#import <Foundation/Foundation.h>
#import <RenderKit/G3DAttributeNode.h>

@class G3DMatrix4f;
@class G3DVector3f;
@class G3DGraphNode;

#define GENERIC     0
#define TRANSLATION 1
#define SCALE       2
#define ROTATION_X  3
#define ROTATION_Y  4
#define ROTATION_Z  5
#define IDENTITY    6

/*
 * Skewing is not supported, as well as there is no custom Pivot point yet!
 */

typedef struct _G3DTransformParam {
  float scale[16];
  float hpr[16];
  float translation[16];
} G3D_TransformParam;

@interface G3DTransform : G3DAttributeNode <NSCopying,NSCoding,G3DSceneCoding>
{
  // Ugly, but I am too lazy to write a private category now...
  @public	
  
  G3D_TransformParam transformation;

  @private
    
  float _scale[3];
  float _hpr[3];
  float _translation[3];

  float m[16];
  float im[16];

  G3DGraphNode *node; // Not retained!

  BOOL shouldScale;
  BOOL shouldHPR;
  BOOL shouldTranslate;

  int _type;
  G3DMatrix4f *_tmpMat;

  BOOL _rotMatIsUsed;
}

//-----------------------------------------------------------------------------
// Class methods
//-----------------------------------------------------------------------------

+ (G3DTransform *)identity;
  // This returns a singleton and should never be altered!!!

+ (G3DTransform *)transform;
+ (G3DTransform *)transformWithScale:(G3DVector3f *)scale;
+ (G3DTransform *)transformWithTranslation:(G3DVector3f *)transl;
+ (G3DTransform *)transformWithHPR:(G3DVector3f *)hpr;
+ (G3DTransform *)transformWithScale:(G3DVector3f *)scale hpr:(G3DVector3f *)hpr translation:(G3DVector3f *)translation;

//-----------------------------------------------------------------------------
// Init
//-----------------------------------------------------------------------------

- (id)init;
  // Designated initialiser, returns an identity transformation

- (id)initWithScale:(G3DVector3f *)scale hpr:(G3DVector3f *)hpr translation:(G3DVector3f *)translation;

//-----------------------------------------------------------------------------
// Transformation
//-----------------------------------------------------------------------------

- (void)transform;
  // Invokes the corresponding GL commands.

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (BOOL)isIdentity;
- (void)makeIdentity;

- (void)setScale:(G3DVector3f *)scaleVec;
- (void)setScaleValues:(float *)scaleValues;
- (void)addScale:(G3DVector3f *)scaleVec;
- (void)addScaleValues:(float *)scaleValues;
- (void)getScaleValues:(float *)scaleValues;
- (void)getScaleMatrix:(G3DMatrix4f **)mat;

- (void)setTranslation:(G3DVector3f *)translationVec;
- (void)setTranslationValues:(float *)translationValues;
- (void)addTranslation:(G3DVector3f *)translationVec;
- (void)addTranslationValues:(float *)translationValues;
- (void)getTranslationValues:(float *)translationValues;
- (void)getTranslationMatrix:(G3DMatrix4f **)mat;

- (void)setHPR:(G3DVector3f *)hprVec;
- (void)setHPRValues:(float *)hprValues;
- (void)addHPR:(G3DVector3f *)hprVec;
- (void)addHPRValues:(float *)hprValues;
- (void)getHPRValues:(float *)hprValues;
- (void)getHPRMatrix:(G3DMatrix4f **)mat;

- (void)setRotationWithElements:(float *)elements;
// If this method is used, you cannot add or get HPR values!

- (void)getTransformationValues:(float *)mat;
- (void)getTransformation:(G3DMatrix4f **)mat;
- (void)getInverseTransformation:(G3DMatrix4f **)mat;

- (void)setNode:(G3DGraphNode *)node;
- (G3DGraphNode *)node;

- (NSString *)description;

//-----------------------------------------------------------------------------
// NSCoding
//-----------------------------------------------------------------------------

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aCoder;

//-----------------------------------------------------------------------------
// NSCopying
//-----------------------------------------------------------------------------

- (id)copyWithZone:(NSZone *)zone;
  // Copy is deep!

//-----------------------------------------------------------------------------
// G3DSceneCoding
//-----------------------------------------------------------------------------

- (void)encodeWithSceneArchiver:(G3DArchiver *)archiver;
- (id)initWithSceneUnarchiver:(G3DUnarchiver *)unarchiver;

@end

extern NSString *G3DTransformInversionException;
extern NSString *G3DTransformException;

#endif




